import re
import pytest

MARKDOWN_PATTERNS = [
    r"(\*\*?|__?)(.*?)\1",        # Matches bold (** or __) and italic (* or _) with proper escaping
    r"`{3}[\s\S]*?`{3}",          # Matches fenced code blocks with triple backticks
    r"`[\s\S]*?`",                # Matches inline code with single backticks
    r"(^|\s)[-+*]\s",             # Matches unordered lists that start with -, +, or *
    r"^\s*#{1,6}\s",              # Matches headers that start with # to ######
    r"^>\s+",                     # Matches blockquotes starting with >
    r"^\d+\.\s",                  # Matches ordered lists starting with 1. or 2. etc
    r"!\[.*?\]\(.*?\)",           # Matches images ![Alt text](URL)
    r"\[.*?\]\(.*?\)"             # Matches links [Link text](URL)
]
markdown_regex = re.compile("|".join(MARKDOWN_PATTERNS), re.MULTILINE)

def assert_is_markdown(received: str):
    is_markdown = markdown_regex.search(received) is not None

    if not is_markdown:
        pytest.fail(f"Expected {received} to include Markdown")


from langchain_core.output_parsers import StrOutputParser
from langchain_core.prompts import PromptTemplate
from langchain_core.messages import HumanMessage
from langchain_openai import ChatOpenAI
from dotenv import load_dotenv

class LLM:
    def __init__(self):
        load_dotenv()
        self.model = ChatOpenAI(model="gpt-4o")

    async def invoke(self, prompt: str, variables: dict) -> str:
        prompt_template = PromptTemplate.from_template(prompt)
        
        # Use the model to generate a response
        chain = prompt_template | self.model | StrOutputParser()
        return chain.invoke(variables)



import re
import pytest

HTML_PATTERNS = [
    r"<(div|span|p|a|img|ul|ol|li|h[1-6]|header|footer|section|article|nav|aside|main|figure|figcaption|blockquote|code|pre|table|thead|tbody|tfoot|tr|td|th|button|input|form|label|textarea|select|option|iframe|video|audio|source|canvas|svg|path|g|circle|rect|line|polygon|polyline|text|tspan|use|symbol|defs|clipPath|mask|pattern|filter|linearGradient|radialGradient|stop|animate|animateTransform|animateMotion|animateColor|desc|title|metadata|foreignObject|switch|symbol|altGlyph|altGlyphDef|altGlyphItem|altGlyphSet|path|view|color-profile|cursor|font|glyph|glyphRef|hkern|vkern|font-face|font-face-format|font-face-name|font-face-src|font-face-uri|missing-glyph|linearGradient|radialGradient|stop|a|altGlyph|animate|circle|clipPath|defs|desc|ellipse|filter|g|image|line|linearGradient|marker|mask|metadata|path|pattern|polygon|polyline|radialGradient|rect|stop|svg|switch|symbol|text|textPath|title|use|view)[\s/>]",  # Matches HTML tags
    r"class\s*=\s*['\"]",  # Matches class attribute
    r"id\s*=\s*['\"]",     # Matches id attribute
    r"style\s*=\s*['\"]"   # Matches style attribute
]
html_regex = re.compile("|".join(HTML_PATTERNS), re.MULTILINE)

def assert_is_html(received: str):
    is_html = html_regex.search(received) is not None

    if not is_html:
        pytest.fail(f"Expected {received} to include HTML")



import re
import pytest

LATEX_PATTERNS = [
    r"\\documentclass",         # Matches document class
    r"\\begin\{[a-zA-Z]+\}",    # Matches \begin{env}
    r"\\end\{[a-zA-Z]+\}",      # Matches \end{env}
    r"\$\$[\s\S]*?\$\$",        # Matches display math with $$
    r"\$[^$]*?\$",              # Matches inline math with $
    r"\\[a-zA-Z]+"              # Matches LaTeX commands
]
latex_regex = re.compile("|".join(LATEX_PATTERNS), re.MULTILINE)

def assert_is_latex(received: str):
    is_latex = latex_regex.search(received) is not None

    if not is_latex:
        pytest.fail(f"Expected {received} to include LaTeX")



import xml.etree.ElementTree as ET
import pytest

def assert_parsable_xml(received: str):
    try:
        ET.fromstring(received)
    except ET.ParseError:
        pytest.fail(f"Expected {received} to be parsable XML")


import pytest
from matchers.parsable.test_parsable_json import assert_parsable_json
from pytest.unitprompt.matchers.parsable.test_parsable_csv import assert_parsable_csv
from pytest.unitprompt.matchers.parsable.test_parsable_xml import assert_parsable_xml
from pytest.unitprompt.matchers.parsable.test_parsable_yaml import assert_parsable_yaml

from matchers.uses.test_uses_html import assert_is_html
from matchers.uses.test_uses_latex import assert_is_latex
from matchers.uses.test_uses_markdown import assert_is_markdown

from matchers.style.test_conciseness import assert_concise_answer, assert_conciseness

from matchers.test_fulfills import assert_fulfills

@pytest.fixture
def parsable_json():
    return assert_parsable_json

@pytest.fixture
def parsable_yaml():
    return assert_parsable_yaml

@pytest.fixture
def parsable_xml():
    return assert_parsable_xml

@pytest.fixture
def parsable_csv():
    return assert_parsable_csv

@pytest.fixture
def uses_html():
    return assert_is_html

@pytest.fixture
def uses_markdown():
    return assert_is_markdown

@pytest.fixture
def uses_latex():
    return assert_is_latex

@pytest.fixture
def is_concise():
    return assert_conciseness

@pytest.fixture
def is_concise_answer():
    return assert_concise_answer

@pytest.fixture
def fulfills():
    return assert_fulfills



import os

def get_prompt(prompt_filename: str) -> str:
    current_dir = os.path.dirname(__file__)  # Directory of the current script
    prompt_path = os.path.join(current_dir, "..", "prompts", prompt_filename + ".txt")
    prompt_path = os.path.abspath(prompt_path)  # Convert to absolute path

    with open(prompt_path, 'r') as f:
        return f.read()



import csv
import io
import pytest

def assert_parsable_csv(received: str):
    try:
        reader = csv.reader(io.StringIO(received))
        first_row = next(reader)
        num_columns = len(first_row)
        for row in reader:
            if len(row) != num_columns:
                pytest.fail(f"Row length mismatch in CSV: expected {num_columns} columns but found {len(row)} in row {row}")
    except csv.Error as e:
        pytest.fail(f"Expected {received} to be parsable CSV, but error occurred: {e}")



import pytest
from unitprompt.llm import LLM
from unitprompt.utils.get_prompt import get_prompt

async def assert_conciseness(received: str):
    llm = LLM()
    llm.model.max_tokens = 1
    
    result = await llm.invoke(get_prompt("concise"), {"received": received})
    if result == "1":
        pass
    else:
        raise pytest.fail(f"Expected {received} to be concise, but it was not")
    
async def assert_concise_answer(received: str, question: str):
    llm = LLM()
    llm.model.max_tokens = 1
    
    result = await llm.invoke(get_prompt("concise_answer"), {"received": received, "question": question})
    print("RESULT:", result)
    if result == "1":
        pass
    else:
        raise pytest.fail(f"Expected {received} to be a concise answer to {question}, but it was not")


import pytest
from unitprompt.llm import LLM


async def assert_fulfills(received: str, prompt: str, goal: str = "true_false"):
    
    complete_prompt = "\nTo test: {received}" 
    
    llm = LLM()
    
    if goal == "true_false":
        llm.model.max_tokens = 1
        complete_prompt += "\nIf true: respond with 1, if false with 0"
        
    result = await llm.invoke(prompt + "\nTo test: {received} \n If true: respond with 1, if false with 0", { "received": received })
    
    if goal == "true_false":
        if result == "1":
            pass
        else:
            raise pytest.fail(f"Expected {received} to be result in a test = 1, but it was not")
    else:
        if result == goal:
            pass
        else:
            raise pytest.fail(f"Expected {received} to be fulfill condition, but it did not")